@@ -32,6 +32,7 @@ const NotificationDaemon = imports.ui.notificationDaemon;
 const WindowAttentionHandler = imports.ui.windowAttentionHandler;
 const Scripting = imports.ui.scripting;
 const CinnamonDBus = imports.ui.cinnamonDBus;
+const LookingGlassDBus = imports.ui.lookingGlassDBus;
 const WindowManager = imports.ui.windowManager;
 const ThemeManager = imports.ui.themeManager;
 const Magnifier = imports.ui.magnifier;
@@ -65,6 +66,7 @@ let notificationDaemon = null;
 let windowAttentionHandler = null;
 let recorder = null;
 let cinnamonDBusService = null;
+let lookingGlassDBusService = null;
 let modalCount = 0;
 let modalActorFocusStack = [];
 let uiGroup = null;
@@ -193,6 +195,7 @@ function start() {
     Gio.DesktopAppInfo.set_desktop_env('GNOME');
 
     cinnamonDBusService = new CinnamonDBus.Cinnamon();
+    lookingGlassDBusService = new LookingGlassDBus.CinnamonLookingGlass();
     // Force a connection now; dbus.js will do this internally
     // if we use its name acquisition stuff but we aren't right
     // now; to do so we'd need to convert from its async calls
@@ -735,12 +738,12 @@ function _log(category, msg) {
                 text += ' ';
         }
     }
-    let out = {timestamp: new Date().getTime(),
+    let out = {timestamp: new Date().getTime().toString(),
                          category: category,
                          message: text };
     _errorLogStack.push(out);
-    if(cinnamonDBusService)
-        cinnamonDBusService.notifyLgLogUpdate();
+    if(lookingGlassDBusService)
+        lookingGlassDBusService.emitLogUpdate();
     if (can_log) lg_log_file.write(renderLogLine(out), null);
 }
 
@@ -805,17 +808,11 @@ function _logInfo(msg) {
 }
 
 function formatTime(d) {
-    function pad(n) { return n < 10 ? '0' + n : n; }
-    return d.getUTCFullYear()+'-'
-        + pad(d.getUTCMonth()+1)+'-'
-        + pad(d.getUTCDate())+'T'
-        + pad(d.getUTCHours())+':'
-        + pad(d.getUTCMinutes())+':'
-        + pad(d.getUTCSeconds())+'Z';
+    return d.toISOString();
 }
 
 function renderLogLine(line) {
-    return line.category + ' t=' + formatTime(new Date(line.timestamp)) + ' ' + line.message + '\n';
+    return line.category + ' t=' + formatTime(new Date(parseInt(line.timestamp))) + ' ' + line.message + '\n';
 }
 
 function logStackTrace(msg) {
@@ -954,21 +951,21 @@ function pushModal(actor, timestamp) {
     let actorDestroyId = actor.connect('destroy', function() {
         let index = _findModal(actor);
         if (index >= 0)
-            modalActorFocusStack.splice(index, 1);
+            popModal(actor);
     });
-    let curFocus = global.stage.get_key_focus();
-    let curFocusDestroyId;
-    if (curFocus != null) {
-        curFocusDestroyId = curFocus.connect('destroy', function() {
-            let index = _findModal(actor);
-            if (index >= 0)
-                modalActorFocusStack[index].actor = null;
+
+    let record = {
+        actor: actor,
+        focus: global.stage.get_key_focus(),
+        destroyId: actorDestroyId
+    };
+    if (record.focus != null) {
+        record.focusDestroyId = record.focus.connect('destroy', function() {
+            record.focus = null;
+            record.focusDestroyId = null;
         });
     }
-    modalActorFocusStack.push({ actor: actor,
-                                focus: curFocus,
-                                destroyId: actorDestroyId,
-                                focusDestroyId: curFocusDestroyId });
+    modalActorFocusStack.push(record);
 
     global.stage.set_key_focus(actor);
     return true;
